/***************************************************
 31/3/2016
 E un programma per il controllo di una macchina ed è quello installato.
 Il controllo del PWM è unico per entrambi i motori
 L'aggiustamento ADJ è individuale per ciascun motore e serve a ridurre
 la velocità di un motore per equipararla a quello dell'altro
 I piedini contrassegnati con D sono quelli del DEVKIT, quello con uscita USB 
 motor1_A = 0;     // D3  pin del DEVKIT
 motor1_B = 2;     // D4
 motor2_A = 14;    // D5
 motor2_B = 12;    // D6
 pwm1     =  3;    // D9
 pwm2     =  1;    // D10
 
 ****************************************************/

// Required libraries
#include <ESP8266WiFi.h>
#include <Wire.h>  
#include <LiquidCrystal_I2C.h> 

#define I2C_ADDR        0x27 // <<- indirizzo del PCF8574
#define Rs_pin          0
#define Rw_pin          1
#define En_pin          2
#define BACKLIGHT_PIN   3
#define D4_pin          4
#define D5_pin          5
#define D6_pin          6
#define D7_pin          7
LiquidCrystal_I2C lcd(I2C_ADDR, En_pin, Rw_pin, Rs_pin, D4_pin, D5_pin, D6_pin, D7_pin);


int spdCurrent1 = 400;  // velocità al primo avvio
int spdCurrent2 = 400;
boolean stopFlag;

// WiFi parameters
const char* ssid = "belkin.44e4";   //  <<- definite la vostra rete WIFI
const char* password = "24f646d9";  //  <<- e la vostra password

// Si crea un' instance di server
WiFiServer server(80);

// Pin
int motor1_A = 0;     // D3  pin del DEVKIT
int motor1_B = 2;     // D4
int motor2_A = 14;    // D5
int motor2_B = 12;    // D6
int pwm1 =      3;    // D9
int pwm2 =      1;    // D10

void DisplayClear(void) {
	int i;
	for (i = 0; i < 4; i++) {
		lcd.setCursor(0, 0);
		lcd.print("                ");
	}
}

//*************************************************
void spdControl(void) {
	if (stopFlag == false) {
		analogWrite(pwm1, spdCurrent1);
		analogWrite(pwm2, spdCurrent2);
	} else {
		analogWrite(pwm1, 0);
		analogWrite(pwm2, 0);
	}
}
//*******************************************************************
void setup() {
	// Viene inizializzato il display
	lcd.begin(16, 4, LCD_5x10DOTS);      // LCD 16x4 e formato caratteri
	lcd.setBacklightPin(BACKLIGHT_PIN, POSITIVE);
	lcd.setBacklight(HIGH);       // accende Bachkight

  // Si definiscono le porte e le condizioni iniziali
	pinMode(motor1_A, OUTPUT);
	digitalWrite(motor1_A, LOW);
	pinMode(motor1_B, OUTPUT);
	digitalWrite(motor1_B, LOW);
	pinMode(motor2_A, OUTPUT);
	digitalWrite(motor2_A, LOW);
	pinMode(motor2_B, OUTPUT);
	digitalWrite(motor2_B, LOW);
	pinMode(pwm1, OUTPUT);
	analogWrite(pwm1, 0);
	pinMode(pwm2, OUTPUT);
	analogWrite(pwm2, 0);

	// Start Seriale
	Serial.begin(115200);
	delay(10);

	WiFi.begin(ssid, password);  // inizializza la connessione WIFI

	// Verifica lo stato fino a connessione avvenuta.
	while (WiFi.status() != WL_CONNECTED) {
		lcd.setCursor(0, 0);
		lcd.print("  CONNESSIONE   ");
		lcd.setCursor(0, 1);
		lcd.print("   IN CORSO     ");
		Serial.print(".");
		delay(500);
	}
 
	DisplayClear();
	lcd.setCursor(0, 0);
	lcd.print("CONNESSO A:     ");
	lcd.setCursor(0, 1);
	lcd.print(ssid);
	lcd.setCursor(0, 2);
	lcd.print("INDIRIZZO IP:   ");
	lcd.setCursor(0, 3);
	lcd.print(WiFi.localIP());
  Serial.print(WiFi.localIP());
	// Start del server
	server.begin();

}

void loop() {

	// Verifica se un client si è connesso
	WiFiClient client = server.available();
	if (!client) {
		return;
	}

	// Attende di ricevere qualche dato
	while (!client.available()) {
		delay(1);
	}

	// Legge la richiesta
	String req = client.readStringUntil('\r');
	client.flush();

	// Analizza le richieste
	// AVANTI
	if (req.indexOf("pin=ON1") != -1) {
		digitalWrite(motor1_A, HIGH);
		digitalWrite(motor1_B, LOW);
		digitalWrite(motor2_A, HIGH);
		digitalWrite(motor2_B, LOW);
		stopFlag = false;
		spdControl();
	}

	// INDIETRO
	else if (req.indexOf("pin=ON2") != -1) {
		digitalWrite(motor1_A, LOW);
		digitalWrite(motor1_B, HIGH);
		digitalWrite(motor2_A, LOW);
		digitalWrite(motor2_B, HIGH);
		stopFlag = false;
		spdControl();
	}

	// FERMA
	else if (req.indexOf("pin=ON3") != -1) {
		digitalWrite(motor1_A, LOW);
		digitalWrite(motor1_B, LOW);
		digitalWrite(motor2_A, LOW);
		digitalWrite(motor2_B, LOW);
		stopFlag = true;
		spdControl();
	}

	// GIRA SINISTRA
	else if (req.indexOf("pin=ON4") != -1) {
		digitalWrite(motor1_A, LOW);
		digitalWrite(motor1_B, LOW);
		digitalWrite(motor2_A, HIGH);
		digitalWrite(motor2_B, LOW);
		stopFlag = false;
		spdControl();
	}

	// GIRA DESTRA
	else if (req.indexOf("pin=OFF4") != -1) {
		digitalWrite(motor1_A, HIGH);
		digitalWrite(motor1_B, LOW);
		digitalWrite(motor2_A, LOW);
		digitalWrite(motor2_B, LOW);
		stopFlag = false;
		spdControl();
	}

	// SPEED ++ AUMENTA
	else if (req.indexOf("pin=ON5") != -1) {
		spdCurrent1 += 50;
		if (spdCurrent1 > 1023)
			spdCurrent1 = 1023;
		spdCurrent2 += 50;
		if (spdCurrent2 > 1023)
			spdCurrent2 = 1023;
		stopFlag = false;
		spdControl();
	}

	// SPEED --  DIMINUISCE 
	else if (req.indexOf("pin=OFF5") != -1) {
		spdCurrent1 -= 50;
		if (spdCurrent1 < 0)
			spdCurrent1 = 0;
		spdCurrent2 -= 50;
		if (spdCurrent2 < 0)
			spdCurrent2 = 0;
		stopFlag = false;
		spdControl();
	}

	// ADJ S  Aggiustamento a sinistra
	else if (req.indexOf("pin=ON6") != -1) {
		spdCurrent1 -= 10;
		if (spdCurrent1 < 0)
			spdCurrent1 = 0;
		stopFlag = false;
		spdControl();
	}
	// ADJ D  Aggiustamento a destra
	else if (req.indexOf("pin=OFF6") != -1) {
		spdCurrent2 -= 10;
		if (spdCurrent2 < 0)
			spdCurrent2 = 0;
		stopFlag = false;
		spdControl();
	}

	client.flush();

	// La stringa costituisce la pagina HTML 
	//Il 200 nella prima riga significa che è OK ha trovato la richiesta
	String buff = "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n";
	buff += "<head>";
	buff += "<title> ESP8266 Car Control </title>";
	buff += "<style>";
	buff +=
			"body { background-color: #cccccc; font-family: Arial, Helvetica, Sans-Serif; Color: #000088; }";
	buff += "</style>";
	buff += "</head>";
	buff += "<body>";
	buff += "<h1 ALIGN=CENTER>ESP8266 CAR CONTROL</h1>";
	buff +=
			"<a href=\"/?pin=ON1\"> <button style=\"background-color: #008000;width:200px;height:70px\"> <font size=\"6\">AVANTI </font> </button> </a>";
	buff +=
			"<a href=\"/?pin=ON2\"> <button style=\"background-color: #cc8000;width:200px;height:70px\"> <font size=\"6\">INDIETRO </font> </button> </a> <br/><br/>";
	buff +=
			"<a href=\"/?pin=ON3\"> <button style=\"background-color: #cc0000;width:200px;height:70px\"> <font size=\"6\">FERMA </font> </button> </a> <br/><br/>";
	buff +=
			"<a href=\"/?pin=ON4\"> <button style=\"background-color: #6699ff;width:200px;height:70px\"> <font size=\"6\">SINISTRA</font> </button> </a>";
	buff +=
			"<a href=\"/?pin=OFF4\"> <button style=\"background-color: #6699ff;width:200px;height:70px\"> <font size=\"6\">DESTRA</font> </button> </a> <br/><br/>";
	buff +=
			"<a href=\"/?pin=ON5\"> <button style=\"background-color: #cccc00;width:200px;height:70px\"> <font size=\"6\">SPEED  ++ </font> </button> </a>";
	buff +=
			"<a href=\"/?pin=OFF5\"> <button style=\"background-color: #cccc00;width:200px;height:70px\"> <font size=\"6\">SPEED  -- </font> </button> </a> <br/><br/>";
	buff +=
			"<a href=\"/?pin=ON6\"> <button style=\"background-color: #cccc00;width:200px;height:70px\"> <font size=\"6\">ADJ-S -- </font> </button> </a>";
	buff +=
			"<a href=\"/?pin=OFF6\"> <button style=\"background-color: #cccc00;width:200px;height:70px\"> <font size=\"6\">ADJ-D -- </font> </button> </a> <br/><br/>";
	buff += "</body>";

	// Invia la risposta al client
	client.print(buff);

	delay(1);
	//  client will viene sconnesso e distrutto l'oggetto cliente 

}
